/* This file has been put into the public domain by its author.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
   OTHER DEALINGS IN THE SOFTWARE.
*/
/* fcmp.c - compare two floats

   AUTHOR: Gregory Pietsch
   
*/

#include "fp.h"
#include "mpi.h"

unsigned
_Fcmp (_FP_T a, _FP_T b, const _FP_METRIC_T * const m)
{
  int sa = 0, sb = 0, c, ca, cb;

  ca = _Classify (a, m);
  cb = _Classify (b, m);
  if (ca == FP_NAN || cb == FP_NAN)
    return 0;			/* unordered */
  /* According to the IEEE 754 standard, negative zero and positive zero 
     should compare as equal with the usual (numerical) comparison operators, 
     like the == operators of C and Java. 

     In those languages, special programming tricks may be needed to 
     distinguish the two values.  */
  if (ca == FP_ZERO && cb == FP_ZERO)
    return _FP_E;
  sa = _Getsign (a, m);
  sb = _Getsign (b, m);
  if (sa != sb)
    return (sa == 0) ? _FP_G : _FP_L;
  c = _Cmp (a, b, (int) m->_Size);
  if (sa)
    return (c < 0) ? _FP_G : ((c > 0) ? _FP_L : _FP_E);
  else
    return (c < 0) ? _FP_L : ((c > 0) ? _FP_G : _FP_E);
}

/* END OF FILE */
