/* This file has been put into the public domain by its author.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR
   OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
   OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef _FP
#define _FP

#include <limits.h>

typedef unsigned char *_FP_T;

typedef union _FP_VALUE_T
{
  unsigned char _X[sizeof (long double)];
  float _F;
  double _D;
  long double _L;
} _FP_VALUE_T;

typedef struct _FP_METRIC_T
{
  int _Size, _Ebits, _Explicit1bit, _Mant_dig, _Dig, _Min_exp, _Max_exp,
    _Min_10_exp, _Max_10_exp;
  _FP_VALUE_T _Max, _Epsilon, _Min, _Inf, _Nan;
} _FP_METRIC_T;

typedef struct _FP_SPLIT_T
{
  unsigned char _S;
  int _E;
  unsigned char _M[sizeof (long double)];
} _FP_SPLIT_T;

/* define __BYTE_ORDER */
#ifndef __BYTE_ORDER
#define __LITTLE_ENDIAN	(0)
#define __BIG_ENDIAN	(1)
#define __PDP_ENDIAN	(2)
#define __MIDDLE_ENDIAN	(3)
#define __BYTE_ORDER	__LITTLE_ENDIAN	/* change if not a PC... */
#endif
#if __BYTE_ORDER == __LITTLE_ENDIAN
#define _F4(_0,_1,_2,_3)\
	_3,_2,_1,_0
#define _F8(_0,_1,_2,_3,_4,_5,_6,_7)\
	_7,_6,_5,_4,_3,_2,_1,_0
#define _F10(_0,_1,_2,_3,_4,_5,_6,_7,_8,_9)\
	_9,_8,_7,_6,_5,_4,_3,_2,_1,_0
#define _F16(_0,_1,_2,_3,_4,_5,_6,_7,_8,_9,_A,_B,_C,_D,_E,_F)\
	_F,_E,_D,_C,_B,_A,_9,_8,_7,_6,_5,_4,_3,_2,_1,_0
#define _BYTE(x,s) ((s)-(x)-1)
#elif __BYTE_ORDER == __BIG_ENDIAN
#define _F4(_0,_1,_2,_3)\
	_0,_1,_2,_3
#define _F8(_0,_1,_2,_3,_4,_5,_6,_7)\
	_0,_1,_2,_3,_4,_5,_6,_7
#define _F10(_0,_1,_2,_3,_4,_5,_6,_7,_8,_9)\
	_0,_1,_2,_3,_4,_5,_6,_7,_8,_9
#define _F16(_0,_1,_2,_3,_4,_5,_6,_7,_8,_9,_A,_B,_C,_D,_E,_F)\
	_0,_1,_2,_3,_4,_5,_6,_7,_8,_9,_A,_B,_C,_D,_E,_F
#define _BYTE(x,s) (x)
#elif __BYTE_ORDER == __PDP_ENDIAN
#define _F4(_0,_1,_2,_3)\
	_2,_3,_0,_1
#define _F8(_0,_1,_2,_3,_4,_5,_6,_7)\
	_6,_7,_4,_5,_2,_3,_0,_1
#define _F10(_0,_1,_2,_3,_4,_5,_6,_7,_8,_9)\
	_8,_9,_6,_7,_4,_5,_2,_3,_0,_1
#define _F16(_0,_1,_2,_3,_4,_5,_6,_7,_8,_9,_A,_B,_C,_D,_E,_F)\
	_E,_F,_C,_D,_A,_B,_8,_9,_6,_7,_4,_5,_2,_3,_0,_1
#define _BYTE(x,s) (((s)-(x)-1)^1)
#elif __BYTE_ORDER == __MIDDLE_ENDIAN
#define _F4(_0,_1,_2,_3)\
	_1,_0,_3,_2
#define _F8(_0,_1,_2,_3,_4,_5,_6,_7)\
	_1,_0,_3,_2,_5,_4,_7,_6
#define _F10(_0,_1,_2,_3,_4,_5,_6,_7,_8,_9)\
	_1,_0,_3,_2,_5,_4,_7,_6,_9,_8
#define _F16(_0,_1,_2,_3,_4,_5,_6,_7,_8,_9,_A,_B,_C,_D,_E,_F)\
	_1,_0,_3,_2,_5,_4,_7,_6,_9,_8,_B,_A,_D,_C,_F,_E
#define _BYTE(x,s) ((x)^1)
#endif

/* The following macros shall be defined for number classification. 
   They represent the mutually-exclusive kinds of floating-point values. 
   They expand to integer constant expressions with distinct values. 
   Additional implementation-defined floating-point classifications, 
   with macro definitions beginning with FP_ and an uppercase letter, 
   may also be specified by the implementation.
*/

#define FP_INFINITE	(1)
#define FP_NAN		(2)
#define FP_NORMAL	(-1)
#define FP_SUBNORMAL	(-2)
#define FP_ZERO		(0)

/* The following macros are for comparison functions: */

#define _FP_G		(4)
#define _FP_L		(2)
#define _FP_E		(1)

#ifdef __cplusplus
extern "C"
{
#endif

_FP_T _Adc(_FP_T c, _FP_T a, _FP_T b, const _FP_METRIC_T *const m);
int _Classify(_FP_T x, const _FP_METRIC_T *const m);
_FP_T _Dvc(_FP_T c, _FP_T a, _FP_T b, const _FP_METRIC_T *const m);
int _Eqc(_FP_T a, _FP_T b, const _FP_METRIC_T *const m);
int _Eqf(_FP_T a, _FP_T b, const _FP_METRIC_T *const m);
unsigned _Fcmp(_FP_T a, _FP_T b, const _FP_METRIC_T *const m);
void _FPsplit (_FP_SPLIT_T * r, _FP_T f, _FP_METRIC_T * m);
int _Gef(_FP_T a, _FP_T b, const _FP_METRIC_T *const m);
int _Getsign(_FP_T x, const _FP_METRIC_T *const m);
int _Gtf(_FP_T a, _FP_T b, const _FP_METRIC_T *const m);
int _Lef(_FP_T a, _FP_T b, const _FP_METRIC_T *const m);
int _Ltf(_FP_T a, _FP_T b, const _FP_METRIC_T *const m);
_FP_T _Mlc(_FP_T c, _FP_T a, _FP_T b, const _FP_METRIC_T *const m);
int _Nec(_FP_T a, _FP_T b, const _FP_METRIC_T *const m);
int _Nef(_FP_T a, _FP_T b, const _FP_METRIC_T *const m);
_FP_T _Ngc(_FP_T c, _FP_T a, const _FP_METRIC_T *const m);
_FP_T _Ngf(_FP_T c, _FP_T a, const _FP_METRIC_T *const m);
_FP_T _Sbc(_FP_T c, _FP_T a, _FP_T b, const _FP_METRIC_T *const m);
int _Setsign(_FP_T x, const _FP_METRIC_T *const m, int sign);

#ifdef __cplusplus
};
#endif

#endif /* _FP */

/* END OF FILE */
